@@ -169,6 +169,8 @@ def summary_cpgs_stats_results_table():
     top3UnionSet = set()
 
     for callname in ToolNameList[:3]:
+        if callname not in top3_cpg_set_dict:
+            continue
         toolSet = top3_cpg_set_dict[callname]
         if not top3JointSet:
             top3JointSet = toolSet
@@ -176,10 +178,10 @@ def summary_cpgs_stats_results_table():
             top3JointSet = top3JointSet.intersection(toolSet)
         top3UnionSet = top3UnionSet.union(toolSet)
 
-    new_row_dict = {f'Total CpG sites by tool cov>={minToolCovCutt}': len(top3JointSet)}
+    new_row_dict = {f'Total CpG sites by tool cov>={minToolCovCutt}': len(top3JointSet) if top3JointSet else None}
     dataset.append(new_row_dict)
 
-    new_row_dict = {f'Total CpG sites by tool cov>={minToolCovCutt}': len(top3UnionSet)}
+    new_row_dict = {f'Total CpG sites by tool cov>={minToolCovCutt}': len(top3UnionSet) if top3UnionSet else None}
     dataset.append(new_row_dict)
 
     # also report top4 joined and union set
@@ -196,10 +198,10 @@ def summary_cpgs_stats_results_table():
             top4JointSet = top4JointSet.intersection(toolSet)
         top4UnionSet = top4UnionSet.union(toolSet)
 
-    new_row_dict = {f'Total CpG sites by tool cov>={minToolCovCutt}': len(top4JointSet)}
+    new_row_dict = {f'Total CpG sites by tool cov>={minToolCovCutt}': len(top4JointSet) if top4JointSet else None}
     dataset.append(new_row_dict)
 
-    new_row_dict = {f'Total CpG sites by tool cov>={minToolCovCutt}': len(top4UnionSet)}
+    new_row_dict = {f'Total CpG sites by tool cov>={minToolCovCutt}': len(top4UnionSet) if top4UnionSet else None}
     dataset.append(new_row_dict)
 
     df = pd.DataFrame(dataset, index=loaded_callname_list +
@@ -376,16 +378,16 @@ def parse_arguments():
     parser.add_argument('--runid', type=str, help="running prefix/output folder name, such as MethCorr-DS_WGBS_2reps",
                         required=True)
     parser.add_argument('--calls', nargs='+',
-                        help='all ONT call results <tool-name>:<file-name> seperated by spaces, tool-name can be Nanopolish, Megalodon, DeepSignal, Guppy, Tombo, METEORE, DeepMod.Cluster',
+                        help='all ONT call results <tool-name>:<file-name> seperated by spaces, tool-name can be Nanopolish, Megalodon, DeepSignal, Guppy, Tombo, METEORE, DeepMod',
                         required=True)
     parser.add_argument('--bgtruth', type=str,
                         help="background truth file <encode-type>:<file-name1>;<file-name1>, encode-type can be 'encode' or 'bismark'",
                         required=True)
     parser.add_argument('--genome-annotation', type=str,
-                        help='genome annotation dir, contain BED files such as singleton, nonsingleton, etc.',
+                        help='genome annotation dir, contain BED files',
                         default=None)
     parser.add_argument('--beddir', type=str,
-                        help="base dir for concordant/discordant BED files generated by read-level analysis, make sure provided dsname is same",
+                        help="base dir for concordant/discordant BED files generated by read-level analysis, make sure the dsname is same",
                         default=None)
     parser.add_argument('--min-bgtruth-cov', type=int, help="cutoff for coverage in bg-truth, default is >=5",
                         default=5)
@@ -550,7 +552,8 @@ if __name__ == '__main__':
 
     top3_cpg_set_dict = defaultdict()
     for callname in ToolNameList[:3]:
-        top3_cpg_set_dict[callname] = set(callresult_dict_cov3[callname].keys())
+        if callname in callresult_dict_cov3:
+            top3_cpg_set_dict[callname] = set(callresult_dict_cov3[callname].keys())
 
     if args.gen_venn:
         logger.info('CPG overlapping analysis')
